#!/usr/bin/perl
# $File: //member/autrijus/Apache-Filter-HanConvert/t/1-basic.t $ $Author: autrijus $
# $Revision: #1 $ $Change: 2677 $ $DateTime: 2002/12/11 16:52:29 $

use Test;
BEGIN { plan tests => 1 }

use Apache::Filter::HanConvert;
ok(1);

__END__
